@@ -567,7 +567,7 @@ peas_engine_garbage_collect (PeasEngine *engine)
 {
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
-  for (guint i = 0; i < G_N_ELEMENTS (loaders); i++)
+  for (guint i = 0; i < G_N_ELEMENTS (engine->loaders); i++)
     {
       LoaderInfo *loader_info = &engine->loaders[i];
 
@@ -875,8 +875,8 @@ create_plugin_loader (int loader_id)
 }
 
 static PeasPluginLoader *
-get_local_plugin_loader (PeasEngine *engine,
-                         int         loader_id)
+get_local_plugin_loader_locked (PeasEngine *engine,
+                                int         loader_id)
 {
   GlobalLoaderInfo *global_loader_info = &loaders[loader_id];
   PeasPluginLoader *loader;
@@ -921,11 +921,14 @@ get_plugin_loader (PeasEngine *engine,
   g_assert (PEAS_IS_ENGINE (engine));
   g_assert (loader_id < PEAS_UTILS_N_LOADERS);
 
-  if (loader_info->loader != NULL || loader_info->failed)
-    return loader_info->loader;
-
   g_mutex_lock (&loaders_lock);
 
+  if (loader_info->loader != NULL || loader_info->failed)
+    {
+      g_mutex_unlock (&loaders_lock);
+      return loader_info->loader;
+    }
+
   if (!loader_info->enabled)
     {
       if (!global_loader_info->enabled)
@@ -950,7 +953,7 @@ get_plugin_loader (PeasEngine *engine,
       return get_plugin_loader (engine, loader_id);
     }
 
-  loader_info->loader = get_local_plugin_loader (engine, loader_id);
+  loader_info->loader = get_local_plugin_loader_locked (engine, loader_id);
 
   if (loader_info->loader == NULL)
     loader_info->failed = TRUE;
@@ -1000,10 +1003,13 @@ peas_engine_enable_loader (PeasEngine *engine,
     }
 
   loader_info = &engine->loaders[loader_id];
-  if (loader_info->enabled || loader_info->failed)
-    return;
 
   g_mutex_lock (&loaders_lock);
+  if (loader_info->enabled || loader_info->failed)
+    {
+      g_mutex_unlock (&loaders_lock);
+      return;
+    }
 
   /* Don't check if the loader failed
    * as we want to warn multiple times
